@@ -19,7 +19,7 @@ class DatabaseCreation(BaseDatabaseCreation):
 			test_db_repr = ''
 			if verbosity >= 2:
 				test_db_repr = " ('%s')" % test_database_name
-			print "Ignoring test database creation for alias '%s'%s..." % (self.connection.alias, test_db_repr)
+			print("Ignoring test database creation for alias '%s'%s..." % (self.connection.alias, test_db_repr))
 		
 		return test_database_name
 	
@@ -29,6 +29,6 @@ class DatabaseCreation(BaseDatabaseCreation):
 			test_db_repr = ''
 			if verbosity >= 2:
 				test_db_repr = " ('%s')" % test_database_name
-			print "No test database to destroy for alias '%s'%s..." % (self.connection.alias, test_db_repr)
+			print("No test database to destroy for alias '%s'%s..." % (self.connection.alias, test_db_repr))
 		self.connection.settings_dict['NAME'] = old_database_name
 
