@@ -25,8 +25,8 @@ class SalesforceManager(manager.Manager):
 		"""
 		Returns a QuerySet which access remote SF objects.
 		"""
-		if router.is_testing(self.db):
-			warning.warn("Incorrectly called SalesforceManager. (probably)", DeprecationWarning)
+		if not router.is_sf_database(self.db):
+			warnings.warn("Incorrectly called SalesforceManager. (probably)", DeprecationWarning)
 			return super(SalesforceManager, self).get_query_set()
 		else:
 			from salesforce.backend import query, compiler
