@@ -99,6 +99,7 @@ def handle_api_exceptions(url, f, *args, **kwargs):
 	log.debug('Request API URL: %s' % url)
 	try:
 		response = f(url, *args, **kwargs_in)
+	# TODO some timeouts can be rarely raised as "SSLError: The read operation timed out"
 	except requests.exceptions.Timeout:
 		raise base.SalesforceError("Timeout, URL=%s" % url)
 	if response.status_code == 401:
@@ -116,27 +117,35 @@ def handle_api_exceptions(url, f, *args, **kwargs):
 	if response.status_code in (200, 201, 204):
 		return response
 
-	# TODO Remove this print after tuning of specific messages. Currently is
-	#      better more than less
+	# TODO Remove this verbose setting after tuning of specific messages.
+	#      Currently it is better more or less.
 	# http://www.salesforce.com/us/developer/docs/api_rest/Content/errorcodes.htm
-	if not getattr(getattr(_cursor, 'query', None), 'debug_silent', False):
-		print("Error (debug details) %s\n%s" % (response.text, response.__dict__))
-	if response.status_code == 404:  # ResourceNotFound
-		raise base.SalesforceError("Couldn't connect to API (404): %s, URL=%s"
-				% (response.text, url))
+	verbose = not getattr(getattr(_cursor, 'query', None), 'debug_silent', False)
 	# Errors are reported in the body
 	data = response.json()[0]
+	if response.status_code == 404:  # ResourceNotFound
+		if (f.__func__.__name__ == 'delete') and data['errorCode'] in (
+				'ENTITY_IS_DELETED', 'INVALID_CROSS_REFERENCE_KEY'):
+			# It is a delete command and the object is in trash bin or
+			# completely deleted or it only could be a valid Id for this type
+			# then is ignored similarly to delete by a classic database query:
+			# DELETE FROM xy WHERE id = 'something_deleted_yet'
+			return None
+		else:
+			# if this Id can not be ever valid.
+			raise base.SalesforceError("Couldn't connect to API (404): %s, URL=%s"
+					% (response.text, url), data, response, verbose)
 	if(data['errorCode'] == 'INVALID_FIELD'):
-		raise base.SalesforceError(data['message'])
+		raise base.SalesforceError(data['message'], data, response, verbose)
 	elif(data['errorCode'] == 'MALFORMED_QUERY'):
-		raise base.SalesforceError(data['message'])
+		raise base.SalesforceError(data['message'], data, response, verbose)
 	elif(data['errorCode'] == 'INVALID_FIELD_FOR_INSERT_UPDATE'):
-		raise base.SalesforceError(data['message'])
+		raise base.SalesforceError(data['message'], data, response, verbose)
 	elif(data['errorCode'] == 'METHOD_NOT_ALLOWED'):
-		raise base.SalesforceError('%s: %s' % (url, data['message']))
+		raise base.SalesforceError('%s: %s' % (url, data['message']), data, response, verbose)
 	# some kind of failed query
 	else:
-		raise base.SalesforceError('%s' % data)
+		raise base.SalesforceError('%s' % data, data, response, verbose)
 
 def prep_for_deserialize(model, record, using):
 	attribs = record.pop('attributes')
@@ -152,15 +161,22 @@ def prep_for_deserialize(model, record, using):
 	fields = dict()
 	for x in model._meta.fields:
 		if not x.primary_key:
-			field_val = record[x.column]
-			db_type = x.db_type(connection=connections[using])
-			if(x.__class__.__name__ == 'DateTimeField' and field_val is not None):
-				d = datetime.datetime.strptime(field_val, SALESFORCE_DATETIME_FORMAT)
-				import pytz
-				d = d.replace(tzinfo=pytz.utc)
-				fields[x.name] = d.strftime(DJANGO_DATETIME_FORMAT)
+			if x.column.endswith('.Type'):
+				# Type of generic foreign key
+				simple_column, _ = x.column.split('.')
+				fields[x.name] = record[simple_column]['Type']
 			else:
-				fields[x.name] = field_val
+				# Normal fields
+				field_val = record[x.column]
+				#db_type = x.db_type(connection=connections[using])
+				if(x.__class__.__name__ == 'DateTimeField' and field_val is not None):
+					d = datetime.datetime.strptime(field_val, SALESFORCE_DATETIME_FORMAT)
+					import pytz
+					d = d.replace(tzinfo=pytz.utc)
+					fields[x.name] = d.strftime(DJANGO_DATETIME_FORMAT)
+				else:
+					fields[x.name] = field_val
+
 
 	return dict(
 		model	= '.'.join([app_label, model.__name__]),
@@ -199,12 +215,10 @@ def extract_values(query):
 
 class SalesforceRawQuerySet(query.RawQuerySet):
 	def __len__(self):
-		if(self.query.cursor is None):
+		if self.query.cursor is None:
 			# force the query
 			self.query.get_columns()
-			return len(self.query.cursor.results)
-		else:
-			return 0;
+		return self.query.cursor.rowcount
 
 class SalesforceQuerySet(query.QuerySet):
 	"""
@@ -241,8 +255,8 @@ class SalesforceRawQuery(RawQuery):
 		if self.cursor is None:
 			self._execute_query()
 		converter = connections[self.using].introspection.table_name_converter
-		if(len(self.cursor.results) > 0):
-			return [converter(col) for col in self.cursor.results[0].keys() if col != 'attributes']
+		if self.cursor.rowcount > 0:
+			return [converter(col) for col in self.cursor.first_row.keys() if col != 'attributes']
 		return ['Id']
 
 	def _execute_query(self):
@@ -295,29 +309,37 @@ class CursorWrapper(object):
 		self.connection = connection
 		self.session = connection.sf_session
 		self.query = query
-		self.results = []
+		# A consistent value is iter([]), but `self.results` can be undefined until execute
+		self.results = None
 		self.rowcount = None
+		self.first_row = None
+
+	def __enter__(self):
+		return self
+
+	def __exit__(self, type, value, traceback):
+		self.close()
 
 	@property
 	def oauth(self):
 		return auth.authenticate(self.settings_dict)
 
-	def execute(self, q, args=None):
+	def execute(self, q, args=()):
 		"""
 		Send a query to the Salesforce API.
 		"""
 		from salesforce.backend import base
 
 		self.rowcount = None
-		if(isinstance(self.query, SalesforceQuery)):
+		if isinstance(self.query, SalesforceQuery) or self.query is None:
 			response = self.execute_select(q, args)
-		elif(isinstance(self.query, SalesforceRawQuery)):
+		elif isinstance(self.query, SalesforceRawQuery):
 			response = self.execute_select(q, args)
-		elif(isinstance(self.query, subqueries.InsertQuery)):
+		elif isinstance(self.query, subqueries.InsertQuery):
 			response = self.execute_insert(self.query)
-		elif(isinstance(self.query, subqueries.UpdateQuery)):
+		elif isinstance(self.query, subqueries.UpdateQuery):
 			response = self.execute_update(self.query)
-		elif(isinstance(self.query, subqueries.DeleteQuery)):
+		elif isinstance(self.query, subqueries.DeleteQuery):
 			response = self.execute_delete(self.query)
 		else:
 			raise base.DatabaseError("Unsupported query: %s" % self.query)
@@ -343,12 +365,14 @@ class CursorWrapper(object):
 			if q.upper().startswith('SELECT COUNT() FROM'):
 				# TODO what about raw query COUNT()
 				# COUNT() queries in SOQL are a special case, as they don't actually return rows
-				self.results = [[self.rowcount]]
+				self.results = iter([[self.rowcount]])
 			else:
-				self.query.first_chunk_len = len(data['records'])
+				if self.query:
+					self.query.first_chunk_len = len(data['records'])
+				self.first_row = data['records'][0] if data['records'] else None
 				self.results = self.query_results(data)
 		else:
-			self.results = []
+			self.results = iter([])
 
 	def execute_select(self, q, args):
 		processed_sql = str(q) % process_args(args)
@@ -412,25 +436,21 @@ class CursorWrapper(object):
 		return handle_api_exceptions(url, self.session.delete, _cursor=self)
 
 	def query_results(self, results):
-		output = []
 		while True:
 			for rec in results['records']:
 				if rec['attributes']['type'] == 'AggregateResult':
 					assert len(rec) -1 == len(list(self.query.aggregate_select.items()))
 					# The 'attributes' info is unexpected for Django within fields.
 					rec = [rec[k] for k, _ in self.query.aggregate_select.items()]
-				output.append(rec)
+				yield rec
 
 			if results['done']:
 				break
-			# TODO convert this method to an iterator providing data after
-			#      every request, not many thousands objects list.
 
 			# see about Retrieving the Remaining SOQL Query Results
-			# http://www.salesforce.com/us/developer/docs/api_rest/Content/dome_query.htm#heading_2_1
+			# http://www.salesforce.com/us/developer/docs/api_rest/Content/dome_query.htm#retrieve_remaining_results_title
 			response = self.query_more(results['nextRecordsUrl'])
 			results = response.json(parse_float=decimal.Decimal)
-		return output
 
 	def __iter__(self):
 		return iter(self.results)
@@ -440,14 +460,16 @@ class CursorWrapper(object):
 		Fetch a single result from a previously executed query.
 		"""
 		try:
-			return self.results.pop(0)
+			return next(self.results)
 		except StopIteration:
 			return None
 
-	def fetchmany(self, size=0):
+	def fetchmany(self, size=None):
 		"""
 		Fetch multiple results from a previously executed query.
 		"""
+		if size is None:
+			size = 200
 		return list(islice(self.results, size))
 
 	def fetchall(self):
