@@ -20,7 +20,7 @@ from django.utils.unittest import skip, skipUnless
 from salesforce.testrunner.example.models import (Account, Contact, Lead, User,
 		BusinessHours, ChargentOrder, CronTrigger,
 		Product, Pricebook, PricebookEntry,
-		GeneralCustomModel, test_custom_db_table, test_custom_db_column)
+		GeneralCustomModel, Note, test_custom_db_table, test_custom_db_column)
 from salesforce import router, DJANGO_15
 from salesforce.backend import sf_alias
 import salesforce
@@ -81,6 +81,8 @@ class BasicSOQLTest(TestCase):
 				"SELECT Id, LastName, FirstName FROM Contact "
 				"LIMIT 2")
 		self.assertEqual(len(contacts), 2)
+		# It had a side effect that the same assert failed second times.
+		self.assertEqual(len(contacts), 2)
 		'%s' % contacts[0].__dict__  # Check that all fields are accessible
 
 	def test_raw_foreignkey_id(self):
@@ -93,7 +95,7 @@ class BasicSOQLTest(TestCase):
 		self.assertEqual(len(contacts), 2)
 		'%s' % contacts[0].__dict__  # Check that all fields are accessible
 		self.assertIn('@', contacts[0].Owner.Email)
-	
+
 	def test_select_all(self):
 		"""
 		Get the first two contact records.
@@ -500,7 +502,7 @@ class BasicSOQLTest(TestCase):
 	def test_z_big_query(self):
 		"""
 		Test a big query that will be splitted to more requests.
-		Test it as late as possible when 
+		Test it as late as possible when
 		"""
 		all_leads = Lead.objects.query_all()
 		leads_list = list(all_leads)
@@ -508,9 +510,24 @@ class BasicSOQLTest(TestCase):
 			self.assertLessEqual(len(leads_list), 2000)
 			print("Not enough Leads accumulated (currently %d including deleted) "
 					"in the last two weeks that are necessary for splitting the "
-					"query into more requests. Number 1001 or 2001 is sure." %
+					"query into more requests. Number 1001 or 2001 is enough." %
 					len(leads_list))
+			self.skipTest("Not enough Leads found for big query test")
 
+	def test_cursor_execute_fetch(self):
+		"""
+		Get results by cursor.execute(...); fetchone(), fetchmany(), fetchall()
+		"""
+		sql = "SELECT Id, LastName, FirstName, OwnerId FROM Contact LIMIT 2"
+		cursor = connections['salesforce'].cursor()
+		cursor.execute(sql)
+		contacts = cursor.fetchall()
+		self.assertEqual(len(contacts), 2)
+		self.assertIn('OwnerId', contacts[0])
+		cursor.execute(sql)
+		self.assertEqual(cursor.fetchone(), contacts[0])
+		self.assertEqual(cursor.fetchmany(), contacts[1:])
+	
 	def test_errors(self):
 		"""
 		Test for improving code coverage.
@@ -526,7 +543,73 @@ class BasicSOQLTest(TestCase):
 		"""
 		# simulate that a request with invalid/expired auth ID re-authenticates
 		# and succeeds.
-		salesforce.auth.oauth_data['salesforce']['access_token'] += 'simulated invalid/expired' 
+		salesforce.auth.oauth_data['salesforce']['access_token'] += 'simulated invalid/expired'
 		self.assertEqual(len(Lead.objects.raw("select Id from Lead limit 1")[0].Id), 18)
 
+	def test_generic_type_field(self):
+		"""
+		Test that a generic foreign key can be filtered by type name and
+		the type name can be referenced.
+		"""
+		test_contact = Contact(FirstName = 'sf_test', LastName='my')
+		test_contact.save()
+		note_1 = Note(title='note for Lead', parent_id=self.test_lead.pk)
+		note_2 = Note(title='note for Contact', parent_id=test_contact.pk)
+		note_1.save()
+		note_2.save()
+		try:
+			self.assertEqual(Note.objects.filter(parent_type='Contact')[0].parent_type, 'Contact')
+			self.assertEqual(Note.objects.filter(parent_type='Lead')[0].parent_type, 'Lead')
+
+			note = Note.objects.filter(parent_type='Contact')[0]
+			parent_model = getattr(salesforce.testrunner.example.models, note.parent_type)
+			parent_object = parent_model.objects.get(pk=note.parent_id)
+			self.assertEqual(parent_object.pk, note.parent_id)
+		finally:
+			note_1.delete()
+			note_2.delete()
+			test_contact.delete()
+
+
+	def test_queryset_values(self):
+		"""
+		Test list of dict qs.values() and list of tuples qs.values_list()
+		"""
+		values = Contact.objects.values()[:2]
+		self.assertEqual(len(values), 2)
+		self.assertIn('FirstName', values[0])
+		self.assertGreater(len(values[0]), 3)
+
+		values = Contact.objects.values('pk', 'FirstName', 'LastName')[:2]
+		self.assertEqual(len(values), 2)
+		self.assertIn('FirstName', values[0])
+		self.assertEqual(len(values[0]), 3)
+
+		values_list = Contact.objects.values_list('pk', 'FirstName', 'LastName')[:2]
+		self.assertEqual(len(values_list), 2)
+		v0 = values[0]
+		self.assertEqual(values_list[0], (v0['pk'], v0['FirstName'], v0['LastName']))
+
+	def test_double_delete(self):
+		"""
+		Test that repeated delete of the same object is ignored the same way
+		like "DELETE FROM Contact WHERE Id='deleted yet'" would do.
+		"""
+		contact = Contact(LastName='sf_test',
+				Owner=User.objects.get(Username=current_user))
+		contact.save()
+		contact_id = contact.pk
+		Contact(pk=contact_id).delete()
+		# Id of a deleted object or a too small valid Id shouldn't raise
+		Contact(pk=contact_id).delete()
+		# Simulate the same with obsoleted oauth session
+		# It is not possible to use salesforce.auth.expire_token() to simulate
+		# expiration because it forces reauhentication before the next request
+		salesforce.auth.oauth_data[sf_alias]['access_token'] = '* something invalid *'
+		Contact(pk=contact_id).delete()
+		# Id of completely deleted item or fake but valid item.
+		Contact(pk='003000000000000AAA').delete()
+		bad_id = '003000000000000AAB' # Id with an incorrect uppercase mask
+		self.assertRaises(salesforce.backend.base.SalesforceError, Contact(pk=bad_id).delete)
+
 	#@skip("Waiting for bug fix")
